@@ -584,6 +584,12 @@ namespace glz
                return;
             }
          }
+         if constexpr (has_runtime_max_string_length<std::decay_t<decltype(ctx)>>) {
+            if (ctx.max_string_length > 0 && n > ctx.max_string_length) [[unlikely]] {
+               ctx.error = error_code::invalid_length;
+               return;
+            }
+         }
          value.resize(n);
          std::memcpy(value.data(), it, n);
          it += n;
@@ -619,6 +625,12 @@ namespace glz
                return;
             }
          }
+         if constexpr (has_runtime_max_string_length<std::decay_t<decltype(ctx)>>) {
+            if (ctx.max_string_length > 0 && n > ctx.max_string_length) [[unlikely]] {
+               ctx.error = error_code::invalid_length;
+               return;
+            }
+         }
 
          if constexpr (string_view_t<T>) {
             value = {it, n};
@@ -812,6 +824,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_array_size > 0 && n > ctx.max_array_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             if constexpr (resizable<T>) {
                value.resize(n);
@@ -861,6 +879,12 @@ namespace glz
                      return 0;
                   }
                }
+               if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+                  if (ctx.max_array_size > 0 && n > ctx.max_array_size) [[unlikely]] {
+                     ctx.error = error_code::invalid_length;
+                     return 0;
+                  }
+               }
 
                if constexpr (resizable<T>) {
                   value.resize(n);
@@ -1005,6 +1029,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_array_size > 0 && n > ctx.max_array_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             if constexpr (resizable<T>) {
                value.resize(n);
@@ -1029,6 +1059,12 @@ namespace glz
                      return;
                   }
                }
+               if constexpr (has_runtime_max_string_length<std::decay_t<decltype(ctx)>>) {
+                  if (ctx.max_string_length > 0 && length > ctx.max_string_length) [[unlikely]] {
+                     ctx.error = error_code::invalid_length;
+                     return;
+                  }
+               }
 
                x.resize(length);
 
@@ -1080,6 +1116,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_array_size > 0 && n > ctx.max_array_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             if constexpr (resizable<T>) {
                value.resize(n);
@@ -1148,6 +1190,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_array_size > 0 && n > ctx.max_array_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             if constexpr (resizable<T>) {
                value.resize(n);
@@ -1312,6 +1360,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_map_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_map_size > 0 && n > ctx.max_map_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
          }
 
          constexpr uint8_t key_tag = beve_key_traits<Key>::key_tag;
@@ -1393,8 +1447,10 @@ namespace glz
                else if constexpr (constructible<T>) {
                   value = meta_construct_v<T>();
                }
-               else if constexpr (check_allocate_raw_pointers(Opts) && std::is_pointer_v<T>) {
-                  value = new std::remove_pointer_t<T>{};
+               else if constexpr (std::is_pointer_v<T> && can_allocate_raw_pointer<Opts, std::decay_t<decltype(ctx)>>) {
+                  if (!try_allocate_raw_pointer<Opts>(value, ctx)) {
+                     return;
+                  }
                }
                else {
                   ctx.error = error_code::invalid_nullable_read;
@@ -1475,7 +1531,7 @@ namespace glz
    struct from<BEVE, T> final
    {
       template <auto Opts>
-         requires(Opts.structs_as_arrays == true)
+         requires(check_structs_as_arrays(Opts) == true)
       static void op(auto&& value, is_context auto&& ctx, auto&& it, auto end)
       {
          if constexpr (reflectable<T>) {
@@ -1526,7 +1582,7 @@ namespace glz
       }
 
       template <auto Opts>
-         requires(Opts.structs_as_arrays == false)
+         requires(check_structs_as_arrays(Opts) == false)
       static void op(auto&& value, is_context auto&& ctx, auto&& it, auto end)
       {
          constexpr uint8_t type = 0; // string key
@@ -1824,8 +1880,8 @@ namespace glz
    [[deprecated("Use read_beve_untagged instead")]] [[nodiscard]] inline error_ctx read_binary_untagged(T&& value,
                                                                                                         Buffer&& buffer)
    {
-      return read<opts{.format = BEVE, .structs_as_arrays = true}>(std::forward<T>(value),
-                                                                   std::forward<Buffer>(buffer));
+      return read<opt_true<opts{.format = BEVE}, structs_as_arrays_opt_tag{}>>(std::forward<T>(value),
+                                                                               std::forward<Buffer>(buffer));
    }
 
    template <read_supported<BEVE> T, class Buffer>
@@ -1833,7 +1889,8 @@ namespace glz
       Buffer&& buffer)
    {
       T value{};
-      const auto pe = read<opts{.format = BEVE, .structs_as_arrays = true}>(value, std::forward<Buffer>(buffer));
+      const auto pe =
+         read<opt_true<opts{.format = BEVE}, structs_as_arrays_opt_tag{}>>(value, std::forward<Buffer>(buffer));
       if (pe) [[unlikely]] {
          return unexpected(pe);
       }
@@ -1844,15 +1901,16 @@ namespace glz
    template <read_supported<BEVE> T, class Buffer>
    [[nodiscard]] inline error_ctx read_beve_untagged(T&& value, Buffer&& buffer)
    {
-      return read<opts{.format = BEVE, .structs_as_arrays = true}>(std::forward<T>(value),
-                                                                   std::forward<Buffer>(buffer));
+      return read<opt_true<opts{.format = BEVE}, structs_as_arrays_opt_tag{}>>(std::forward<T>(value),
+                                                                               std::forward<Buffer>(buffer));
    }
 
    template <read_supported<BEVE> T, class Buffer>
    [[nodiscard]] inline expected<T, error_ctx> read_beve_untagged(Buffer&& buffer)
    {
       T value{};
-      const auto pe = read<opts{.format = BEVE, .structs_as_arrays = true}>(value, std::forward<Buffer>(buffer));
+      const auto pe =
+         read<opt_true<opts{.format = BEVE}, structs_as_arrays_opt_tag{}>>(value, std::forward<Buffer>(buffer));
       if (pe) [[unlikely]] {
          return unexpected(pe);
       }
@@ -1862,7 +1920,7 @@ namespace glz
    template <auto Opts = opts{}, read_supported<BEVE> T>
    [[nodiscard]] inline error_ctx read_file_beve_untagged(T& value, const std::string& file_name, auto&& buffer)
    {
-      return read_file_beve<opt_true<Opts, &opts::structs_as_arrays>>(value, file_name, buffer);
+      return read_file_beve<opt_true<Opts, structs_as_arrays_opt_tag{}>>(value, file_name, buffer);
    }
 
    // ===== Delimited BEVE support for multiple objects in one buffer =====
@@ -1961,8 +2019,14 @@ namespace glz
    //      if (!result) break;
    //      offset += *result;  // correctly advances past delimiter + value
    //   }
+   //
+   // Overload with context for runtime options (e.g., max_string_length, max_array_size):
+   //   my_context ctx{};
+   //   ctx.max_string_length = 1024;
+   //   auto result = glz::read_beve_at<glz::opts{}>(value, buffer, offset, ctx);
    template <auto Opts = opts{}, read_supported<BEVE> T, class Buffer>
-   [[nodiscard]] glz::expected<size_t, error_ctx> read_beve_at(T& value, Buffer&& buffer, size_t offset = 0)
+   [[nodiscard]] glz::expected<size_t, error_ctx> read_beve_at(T& value, Buffer&& buffer, size_t offset,
+                                                               is_context auto&& ctx)
    {
       static_assert(sizeof(decltype(*buffer.data())) == 1);
 
@@ -1970,7 +2034,6 @@ namespace glz
          return glz::unexpected(error_ctx{0, error_code::unexpected_end});
       }
 
-      context ctx{};
       auto it = reinterpret_cast<const char*>(buffer.data()) + offset;
       auto end = reinterpret_cast<const char*>(buffer.data()) + buffer.size();
       auto start = it;
@@ -1990,4 +2053,11 @@ namespace glz
 
       return size_t(it - start); // total bytes consumed from offset (delimiter + value)
    }
+
+   template <auto Opts = opts{}, read_supported<BEVE> T, class Buffer>
+   [[nodiscard]] glz::expected<size_t, error_ctx> read_beve_at(T& value, Buffer&& buffer, size_t offset = 0)
+   {
+      context ctx{};
+      return read_beve_at<Opts>(value, std::forward<Buffer>(buffer), offset, ctx);
+   }
 }
