@@ -564,6 +564,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_string_length<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_string_length > 0 && length > ctx.max_string_length) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             if constexpr (string_view_t<T>) {
                value = {reinterpret_cast<const char*>(it), static_cast<size_t>(length)};
@@ -665,6 +671,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_array_size > 0 && length > ctx.max_array_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             value.resize(static_cast<size_t>(length));
             std::memcpy(value.data(), it, length);
@@ -760,6 +772,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_array_size > 0 && length > ctx.max_array_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             value.resize(static_cast<size_t>(length));
             std::memcpy(value.data(), it, length);
@@ -842,6 +860,12 @@ namespace glz
                         return;
                      }
                   }
+                  if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+                     if (ctx.max_array_size > 0 && count > ctx.max_array_size) [[unlikely]] {
+                        ctx.error = error_code::invalid_length;
+                        return;
+                     }
+                  }
 
                   if constexpr (resizable<T>) {
                      value.resize(count);
@@ -983,6 +1007,12 @@ namespace glz
                         return;
                      }
                   }
+                  if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+                     if (ctx.max_array_size > 0 && count > ctx.max_array_size) [[unlikely]] {
+                        ctx.error = error_code::invalid_length;
+                        return;
+                     }
+                  }
 
                   if constexpr (resizable<T>) {
                      value.resize(count);
@@ -1110,6 +1140,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_array_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_array_size > 0 && count > ctx.max_array_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             if constexpr (resizable<T>) {
                value.resize(static_cast<size_t>(count));
@@ -1207,6 +1243,12 @@ namespace glz
                   return;
                }
             }
+            if constexpr (has_runtime_max_map_size<std::decay_t<decltype(ctx)>>) {
+               if (ctx.max_map_size > 0 && count > ctx.max_map_size) [[unlikely]] {
+                  ctx.error = error_code::invalid_length;
+                  return;
+               }
+            }
 
             for (size_t i = 0; i < count; ++i) {
                Key key{};
@@ -1543,8 +1585,10 @@ namespace glz
                else if constexpr (constructible<T>) {
                   value = meta_construct_v<T>();
                }
-               else if constexpr (check_allocate_raw_pointers(Opts) && std::is_pointer_v<T>) {
-                  value = new std::remove_pointer_t<T>{};
+               else if constexpr (std::is_pointer_v<T> && can_allocate_raw_pointer<Opts, std::decay_t<decltype(ctx)>>) {
+                  if (!try_allocate_raw_pointer<Opts>(value, ctx)) {
+                     return;
+                  }
                }
                else {
                   ctx.error = error_code::invalid_nullable_read;
