@@ -256,6 +256,20 @@ namespace glz
    template <class T>
    concept mimics_str_t = has_mimic<T> && str_t<mimic_type<T>>;
 
+   // Concept: does T's mimic type satisfy num_t?
+   // This allows checking if a custom type mimics numeric behavior.
+   // Enables proper variant type deduction for custom numeric types.
+   template <class T>
+   concept mimics_num_t = has_mimic<T> && num_t<mimic_type<T>>;
+
+   // Concept: does T's mimic type satisfy bool_t?
+   // This allows checking if a custom type mimics boolean behavior.
+   template <class T>
+   concept mimics_bool_t = has_mimic<T> && bool_t<mimic_type<T>>;
+
+   // Note: custom_num_t, custom_str_t, custom_bool_t concepts are defined in core/custom_meta.hpp
+   // (requires custom_t from wrappers.hpp which is not included here)
+
    template <class T>
    concept readable_map_t = !custom_read<T> && !meta_value_t<T> && !str_t<T> && range<T> && pair_t<range_value_t<T>> &&
                             map_subscriptable<std::decay_t<T>>;
@@ -362,8 +376,19 @@ namespace glz
       { *t };
    };
 
+   // Detect function pointers and function references (which should not be treated as nullable)
    template <class T>
-   concept nullable_like = nullable_t<T> && (!is_expected<T> && !std::is_array_v<T>);
+   concept is_function_ptr_or_ref =
+      std::is_function_v<std::remove_cvref_t<T>> || // function reference: void(&)(int)
+      (std::is_pointer_v<std::remove_cvref_t<T>> &&
+       std::is_function_v<std::remove_pointer_t<std::remove_cvref_t<T>>>); // function pointer: void(*)(int)
+
+   // Combined concept for any function pointer type (member or non-member)
+   template <class T>
+   concept is_any_function_ptr = is_member_function_pointer<T> || is_function_ptr_or_ref<T>;
+
+   template <class T>
+   concept nullable_like = nullable_t<T> && !is_expected<T> && !std::is_array_v<T> && !is_function_ptr_or_ref<T>;
 
    // For optional like types that cannot overload `operator bool()`
    template <class T>
