@@ -6,6 +6,9 @@
 #include <cstdint>
 #include <type_traits>
 
+#include "glaze/core/context.hpp"
+#include "glaze/core/optimization_level.hpp"
+#include "glaze/util/inline.hpp"
 #include "glaze/util/type_traits.hpp"
 
 namespace glz
@@ -88,18 +91,9 @@ namespace glz
       bool skip_null_members = true; // Skip writing out params in an object if the value is null
       bool prettify = false; // Write out prettified JSON
       bool minified = false; // Require minified input for JSON, which results in faster read performance
-      char indentation_char = ' '; // Prettified JSON indentation char
-      uint8_t indentation_width = 3; // Prettified JSON indentation size
-      bool new_lines_in_arrays = true; // Whether prettified arrays should have new lines for each element
       bool error_on_missing_keys = false; // Require all non nullable keys to be present in the object. Use
                                           // skip_null_members = false to require nullable members
 
-      bool quoted_num = false; // treat numbers as quoted or array-like types as having quoted numbers
-      bool number = false; // treats all types like std::string as numbers: read/write these quoted numbers
-      bool raw = false; // write out string like values without quotes
-      bool raw_string = false; // do not decode/encode escaped characters for strings (improves read/write performance)
-      bool structs_as_arrays = false; // Handle structs (reading/writing) without keys, which applies
-
       bool partial_read =
          false; // Reads into the deepest structural object and then exits without parsing the rest of the input
 
@@ -142,8 +136,8 @@ namespace glz
    // If full validation should be performed on skipped values
 
    // ---
-   // bool write_member_functions = false;
-   // If member function pointers should be serialized when provided in glz::meta
+   // bool write_function_pointers = false;
+   // If function pointers (member and non-member) should be serialized when provided in glz::meta
 
    // ---
    // bool validate_trailing_whitespace = false;
@@ -208,6 +202,12 @@ namespace glz
    // Trades O(1) hash lookup for O(N) linear search - faster for small structs (< ~8 fields)
    // due to cache effects, and much smaller binaries for embedded systems.
 
+   // ---
+   // optimization_level optimization_level = optimization_level::normal;
+   // Controls speed vs binary size tradeoff. See glaze/core/optimization_level.hpp for details.
+   // Levels: size, normal (default)
+   // Use preset struct: glz::opts_size
+
    // ---
    // size_t max_string_length = 0;
    // Maximum length for string allocations when reading. 0 means no limit (default).
@@ -230,6 +230,48 @@ namespace glz
    // Enable this option only when you are prepared to manually manage the allocated memory.
    // Works with JSON, BEVE, CBOR, and MSGPACK formats.
 
+   // ---
+   // char indentation_char = ' ';
+   // Prettified JSON indentation character. Use '\t' for tabs.
+
+   // ---
+   // uint8_t indentation_width = 3;
+   // Prettified JSON indentation size (number of indentation_char per level).
+
+   // ---
+   // bool new_lines_in_arrays = true;
+   // Whether prettified arrays should have new lines for each element.
+   // Set to false for more compact array output.
+
+   // ---
+   // bool quoted_num = false;
+   // Treat numbers as quoted strings. When writing, numbers are wrapped in quotes: 123 -> "123"
+   // When reading, quoted numbers are parsed: "123" -> 123
+   // Useful for APIs that require numeric values as strings.
+
+   // ---
+   // bool string_as_number = false;
+   // Treat string types (like std::string) as numbers during serialization.
+   // When writing, strings are output without quotes. When reading, unquoted values are read into strings.
+   // Useful for reading/writing numeric strings without quote overhead.
+
+   // ---
+   // bool unquoted = false;
+   // Write string-like values without surrounding quotes.
+   // The string content is still escape-processed unless raw_string is also true.
+
+   // ---
+   // bool raw_string = false;
+   // Skip escape sequence encoding/decoding for strings.
+   // Improves read/write performance when strings are known to not contain escape characters.
+   // Can be combined with 'raw' to write completely unprocessed string content.
+
+   // ---
+   // bool structs_as_arrays = false;
+   // Serialize/deserialize structs as arrays without field keys.
+   // Useful for binary formats or when field names add unnecessary overhead.
+   // Order of fields is determined by the glz::meta definition.
+
    struct append_arrays_opt_tag
    {};
 
@@ -239,6 +281,27 @@ namespace glz
    struct escape_control_characters_opt_tag
    {};
 
+   struct quoted_num_opt_tag
+   {};
+
+   struct string_as_number_opt_tag
+   {};
+
+   struct unquoted_opt_tag
+   {};
+
+   struct raw_string_opt_tag
+   {};
+
+   struct structs_as_arrays_opt_tag
+   {};
+
+   // Helper for deprecated option static_asserts (dependent false pattern)
+   template <class>
+   inline constexpr bool deprecated_opts_raw = false;
+   template <class>
+   inline constexpr bool deprecated_opts_number = false;
+
    template <auto member_ptr>
    concept is_append_arrays_tag = std::same_as<std::decay_t<decltype(member_ptr)>, append_arrays_opt_tag>;
 
@@ -249,6 +312,21 @@ namespace glz
    concept is_escape_control_characters_tag =
       std::same_as<std::decay_t<decltype(member_ptr)>, escape_control_characters_opt_tag>;
 
+   template <auto member_ptr>
+   concept is_quoted_num_tag = std::same_as<std::decay_t<decltype(member_ptr)>, quoted_num_opt_tag>;
+
+   template <auto member_ptr>
+   concept is_string_as_number_tag = std::same_as<std::decay_t<decltype(member_ptr)>, string_as_number_opt_tag>;
+
+   template <auto member_ptr>
+   concept is_unquoted_tag = std::same_as<std::decay_t<decltype(member_ptr)>, unquoted_opt_tag>;
+
+   template <auto member_ptr>
+   concept is_raw_string_tag = std::same_as<std::decay_t<decltype(member_ptr)>, raw_string_opt_tag>;
+
+   template <auto member_ptr>
+   concept is_structs_as_arrays_tag = std::same_as<std::decay_t<decltype(member_ptr)>, structs_as_arrays_opt_tag>;
+
    consteval bool check_validate_skipped(auto&& Opts)
    {
       if constexpr (requires { Opts.validate_skipped; }) {
@@ -279,10 +357,10 @@ namespace glz
       }
    }
 
-   consteval bool check_write_member_functions(auto&& Opts)
+   consteval bool check_write_function_pointers(auto&& Opts)
    {
-      if constexpr (requires { Opts.write_member_functions; }) {
-         return Opts.write_member_functions;
+      if constexpr (requires { Opts.write_function_pointers; }) {
+         return Opts.write_function_pointers;
       }
       else {
          return false;
@@ -409,6 +487,84 @@ namespace glz
       }
    }
 
+   consteval bool check_quoted_num(auto&& Opts)
+   {
+      if constexpr (requires { Opts.quoted_num; }) {
+         return Opts.quoted_num;
+      }
+      else {
+         return false;
+      }
+   }
+
+   consteval bool check_string_as_number(auto&& Opts)
+   {
+      if constexpr (requires { Opts.string_as_number; }) {
+         return Opts.string_as_number;
+      }
+      else if constexpr (requires { Opts.number; }) {
+         static_assert(deprecated_opts_number<std::decay_t<decltype(Opts)>>,
+                       "\n\n"
+                       "  [glaze] Deprecated: 'number' option has been renamed to 'string_as_number'\n"
+                       "  \n"
+                       "  Please update your custom opts struct:\n"
+                       "  \n"
+                       "    // OLD (deprecated):\n"
+                       "    struct my_opts : glz::opts {\n"
+                       "       bool number = true;\n"
+                       "    };\n"
+                       "  \n"
+                       "    // NEW:\n"
+                       "    struct my_opts : glz::opts {\n"
+                       "       bool string_as_number = true;\n"
+                       "    };\n"
+                       "\n");
+         return Opts.number;
+      }
+      else {
+         return false;
+      }
+   }
+
+   consteval bool check_unquoted(auto&& Opts)
+   {
+      if constexpr (requires { Opts.unquoted; }) {
+         return Opts.unquoted;
+      }
+      else if constexpr (requires { Opts.raw; }) {
+         static_assert(deprecated_opts_raw<std::decay_t<decltype(Opts)>>,
+                       "\n\n"
+                       "  [glaze] Deprecated: 'raw' option has been renamed to 'unquoted'\n"
+                       "  \n"
+                       "  Please update your custom opts struct:\n"
+                       "  \n"
+                       "    // OLD (deprecated):\n"
+                       "    struct my_opts : glz::opts {\n"
+                       "       bool raw = true;\n"
+                       "    };\n"
+                       "  \n"
+                       "    // NEW:\n"
+                       "    struct my_opts : glz::opts {\n"
+                       "       bool unquoted = true;\n"
+                       "    };\n"
+                       "\n");
+         return Opts.raw;
+      }
+      else {
+         return false;
+      }
+   }
+
+   consteval bool check_structs_as_arrays(auto&& Opts)
+   {
+      if constexpr (requires { Opts.structs_as_arrays; }) {
+         return Opts.structs_as_arrays;
+      }
+      else {
+         return false;
+      }
+   }
+
    consteval uint8_t check_layout(auto&& Opts)
    {
       if constexpr (requires { Opts.layout; }) {
@@ -419,6 +575,36 @@ namespace glz
       }
    }
 
+   consteval char check_indentation_char(auto&& Opts)
+   {
+      if constexpr (requires { Opts.indentation_char; }) {
+         return Opts.indentation_char;
+      }
+      else {
+         return ' ';
+      }
+   }
+
+   consteval uint8_t check_indentation_width(auto&& Opts)
+   {
+      if constexpr (requires { Opts.indentation_width; }) {
+         return Opts.indentation_width;
+      }
+      else {
+         return 3;
+      }
+   }
+
+   consteval bool check_new_lines_in_arrays(auto&& Opts)
+   {
+      if constexpr (requires { Opts.new_lines_in_arrays; }) {
+         return Opts.new_lines_in_arrays;
+      }
+      else {
+         return true;
+      }
+   }
+
    consteval float_precision check_float_max_write_precision(auto&& Opts)
    {
       if constexpr (requires { Opts.float_max_write_precision; }) {
@@ -455,6 +641,10 @@ namespace glz
          return Opts.linear_search;
       }
       else {
+         // In size mode, default to linear search (no hash tables)
+         if constexpr (requires { Opts.optimization_level; }) {
+            return Opts.optimization_level == optimization_level::size;
+         }
          return false;
       }
    }
@@ -499,6 +689,47 @@ namespace glz
       }
    }
 
+   consteval optimization_level check_optimization_level(auto&& Opts)
+   {
+      if constexpr (requires { Opts.optimization_level; }) {
+         return Opts.optimization_level;
+      }
+      else {
+         return optimization_level::normal;
+      }
+   }
+
+   consteval bool is_size_optimized(auto&& Opts) { return check_optimization_level(Opts) == optimization_level::size; }
+
+   // Check if raw pointer allocation is possible (either compile-time or runtime option available)
+   template <auto Opts, class Ctx>
+   concept can_allocate_raw_pointer = check_allocate_raw_pointers(Opts) || has_runtime_allocate_raw_pointers<Ctx>;
+
+   // Helper to attempt allocation of a null raw pointer during deserialization.
+   // Compile-time allocate_raw_pointers option takes precedence over runtime context.
+   // Returns true on success, false on failure (with ctx.error set to invalid_nullable_read).
+   // Only call this when can_allocate_raw_pointer<Opts, Ctx> is satisfied.
+   template <auto Opts, class Ptr, class Ctx>
+      requires(std::is_pointer_v<std::remove_cvref_t<Ptr>> && can_allocate_raw_pointer<Opts, std::decay_t<Ctx>>)
+   GLZ_ALWAYS_INLINE constexpr bool try_allocate_raw_pointer(Ptr& value, Ctx& ctx) noexcept
+   {
+      using PtrType = std::remove_cvref_t<Ptr>;
+      using PointedType = std::remove_pointer_t<PtrType>;
+
+      if constexpr (check_allocate_raw_pointers(Opts)) {
+         value = new PointedType{};
+         return true;
+      }
+      else if constexpr (has_runtime_allocate_raw_pointers<std::decay_t<Ctx>>) {
+         if (ctx.allocate_raw_pointers) {
+            value = new PointedType{};
+            return true;
+         }
+         ctx.error = error_code::invalid_nullable_read;
+         return false;
+      }
+   }
+
    consteval bool check_opening_handled(auto&& o) { return o.internal & uint32_t(opts_internal::opening_handled); }
 
    consteval bool check_closing_handled(auto&& o) { return o.internal & uint32_t(opts_internal::closing_handled); }
@@ -681,6 +912,96 @@ namespace glz
             return opts_escape_control_characters{{Opts}, static_cast<bool>(value)};
          }
       }
+      else if constexpr (is_quoted_num_tag<member_ptr>) {
+         if constexpr (requires { Opts.quoted_num; }) {
+            auto ret = Opts;
+            ret.quoted_num = static_cast<bool>(value);
+            return ret;
+         }
+         else {
+            struct opts_quoted_num : std::decay_t<decltype(Opts)>
+            {
+               bool quoted_num{};
+            };
+            return opts_quoted_num{{Opts}, static_cast<bool>(value)};
+         }
+      }
+      else if constexpr (is_string_as_number_tag<member_ptr>) {
+         if constexpr (requires { Opts.string_as_number; }) {
+            auto ret = Opts;
+            ret.string_as_number = static_cast<bool>(value);
+            return ret;
+         }
+         else if constexpr (requires { Opts.number; }) {
+            static_assert(deprecated_opts_number<std::decay_t<decltype(Opts)>>,
+                          "\n\n"
+                          "  [glaze] Deprecated: 'number' option has been renamed to 'string_as_number'\n"
+                          "  Please rename 'bool number' to 'bool string_as_number' in your custom opts struct.\n"
+                          "\n");
+            auto ret = Opts;
+            ret.number = static_cast<bool>(value);
+            return ret;
+         }
+         else {
+            struct opts_string_as_number : std::decay_t<decltype(Opts)>
+            {
+               bool string_as_number{};
+            };
+            return opts_string_as_number{{Opts}, static_cast<bool>(value)};
+         }
+      }
+      else if constexpr (is_unquoted_tag<member_ptr>) {
+         if constexpr (requires { Opts.unquoted; }) {
+            auto ret = Opts;
+            ret.unquoted = static_cast<bool>(value);
+            return ret;
+         }
+         else if constexpr (requires { Opts.raw; }) {
+            static_assert(deprecated_opts_raw<std::decay_t<decltype(Opts)>>,
+                          "\n\n"
+                          "  [glaze] Deprecated: 'raw' option has been renamed to 'unquoted'\n"
+                          "  Please rename 'bool raw' to 'bool unquoted' in your custom opts struct.\n"
+                          "\n");
+            auto ret = Opts;
+            ret.raw = static_cast<bool>(value);
+            return ret;
+         }
+         else {
+            struct opts_unquoted : std::decay_t<decltype(Opts)>
+            {
+               bool unquoted{};
+            };
+            return opts_unquoted{{Opts}, static_cast<bool>(value)};
+         }
+      }
+      else if constexpr (is_raw_string_tag<member_ptr>) {
+         if constexpr (requires { Opts.raw_string; }) {
+            auto ret = Opts;
+            ret.raw_string = static_cast<bool>(value);
+            return ret;
+         }
+         else {
+            struct opts_raw_string : std::decay_t<decltype(Opts)>
+            {
+               bool raw_string{};
+            };
+            return opts_raw_string{{Opts}, static_cast<bool>(value)};
+         }
+      }
+      else if constexpr (is_structs_as_arrays_tag<member_ptr>) {
+         if constexpr (requires { Opts.structs_as_arrays; }) {
+            auto ret = Opts;
+            ret.structs_as_arrays = static_cast<bool>(value);
+            return ret;
+         }
+         else {
+            struct opts_structs_as_arrays : std::decay_t<decltype(Opts)>
+            {
+               bool structs_as_arrays{};
+            };
+            return opts_structs_as_arrays{{Opts}, static_cast<bool>(value)};
+         }
+      }
       else {
          auto ret = Opts;
          ret.*member_ptr = value;
@@ -733,6 +1054,96 @@ namespace glz
             return opts_escape_control_characters{{Opts}};
          }
       }
+      else if constexpr (is_quoted_num_tag<member_ptr>) {
+         if constexpr (requires { Opts.quoted_num; }) {
+            auto ret = Opts;
+            ret.quoted_num = true;
+            return ret;
+         }
+         else {
+            struct opts_quoted_num : std::decay_t<decltype(Opts)>
+            {
+               bool quoted_num = true;
+            };
+            return opts_quoted_num{{Opts}};
+         }
+      }
+      else if constexpr (is_string_as_number_tag<member_ptr>) {
+         if constexpr (requires { Opts.string_as_number; }) {
+            auto ret = Opts;
+            ret.string_as_number = true;
+            return ret;
+         }
+         else if constexpr (requires { Opts.number; }) {
+            static_assert(deprecated_opts_number<std::decay_t<decltype(Opts)>>,
+                          "\n\n"
+                          "  [glaze] Deprecated: 'number' option has been renamed to 'string_as_number'\n"
+                          "  Please rename 'bool number' to 'bool string_as_number' in your custom opts struct.\n"
+                          "\n");
+            auto ret = Opts;
+            ret.number = true;
+            return ret;
+         }
+         else {
+            struct opts_string_as_number : std::decay_t<decltype(Opts)>
+            {
+               bool string_as_number = true;
+            };
+            return opts_string_as_number{{Opts}};
+         }
+      }
+      else if constexpr (is_unquoted_tag<member_ptr>) {
+         if constexpr (requires { Opts.unquoted; }) {
+            auto ret = Opts;
+            ret.unquoted = true;
+            return ret;
+         }
+         else if constexpr (requires { Opts.raw; }) {
+            static_assert(deprecated_opts_raw<std::decay_t<decltype(Opts)>>,
+                          "\n\n"
+                          "  [glaze] Deprecated: 'raw' option has been renamed to 'unquoted'\n"
+                          "  Please rename 'bool raw' to 'bool unquoted' in your custom opts struct.\n"
+                          "\n");
+            auto ret = Opts;
+            ret.raw = true;
+            return ret;
+         }
+         else {
+            struct opts_unquoted : std::decay_t<decltype(Opts)>
+            {
+               bool unquoted = true;
+            };
+            return opts_unquoted{{Opts}};
+         }
+      }
+      else if constexpr (is_raw_string_tag<member_ptr>) {
+         if constexpr (requires { Opts.raw_string; }) {
+            auto ret = Opts;
+            ret.raw_string = true;
+            return ret;
+         }
+         else {
+            struct opts_raw_string : std::decay_t<decltype(Opts)>
+            {
+               bool raw_string = true;
+            };
+            return opts_raw_string{{Opts}};
+         }
+      }
+      else if constexpr (is_structs_as_arrays_tag<member_ptr>) {
+         if constexpr (requires { Opts.structs_as_arrays; }) {
+            auto ret = Opts;
+            ret.structs_as_arrays = true;
+            return ret;
+         }
+         else {
+            struct opts_structs_as_arrays : std::decay_t<decltype(Opts)>
+            {
+               bool structs_as_arrays = true;
+            };
+            return opts_structs_as_arrays{{Opts}};
+         }
+      }
       else {
          auto ret = Opts;
          ret.*member_ptr = true;
@@ -776,6 +1187,76 @@ namespace glz
             return Opts;
          }
       }
+      else if constexpr (is_quoted_num_tag<member_ptr>) {
+         if constexpr (requires { Opts.quoted_num; }) {
+            auto ret = Opts;
+            ret.quoted_num = false;
+            return ret;
+         }
+         else {
+            return Opts;
+         }
+      }
+      else if constexpr (is_string_as_number_tag<member_ptr>) {
+         if constexpr (requires { Opts.string_as_number; }) {
+            auto ret = Opts;
+            ret.string_as_number = false;
+            return ret;
+         }
+         else if constexpr (requires { Opts.number; }) {
+            static_assert(deprecated_opts_number<std::decay_t<decltype(Opts)>>,
+                          "\n\n"
+                          "  [glaze] Deprecated: 'number' option has been renamed to 'string_as_number'\n"
+                          "  Please rename 'bool number' to 'bool string_as_number' in your custom opts struct.\n"
+                          "\n");
+            auto ret = Opts;
+            ret.number = false;
+            return ret;
+         }
+         else {
+            return Opts;
+         }
+      }
+      else if constexpr (is_unquoted_tag<member_ptr>) {
+         if constexpr (requires { Opts.unquoted; }) {
+            auto ret = Opts;
+            ret.unquoted = false;
+            return ret;
+         }
+         else if constexpr (requires { Opts.raw; }) {
+            static_assert(deprecated_opts_raw<std::decay_t<decltype(Opts)>>,
+                          "\n\n"
+                          "  [glaze] Deprecated: 'raw' option has been renamed to 'unquoted'\n"
+                          "  Please rename 'bool raw' to 'bool unquoted' in your custom opts struct.\n"
+                          "\n");
+            auto ret = Opts;
+            ret.raw = false;
+            return ret;
+         }
+         else {
+            return Opts;
+         }
+      }
+      else if constexpr (is_raw_string_tag<member_ptr>) {
+         if constexpr (requires { Opts.raw_string; }) {
+            auto ret = Opts;
+            ret.raw_string = false;
+            return ret;
+         }
+         else {
+            return Opts;
+         }
+      }
+      else if constexpr (is_structs_as_arrays_tag<member_ptr>) {
+         if constexpr (requires { Opts.structs_as_arrays; }) {
+            auto ret = Opts;
+            ret.structs_as_arrays = false;
+            return ret;
+         }
+         else {
+            return Opts;
+         }
+      }
       else {
          auto ret = Opts;
          ret.*member_ptr = false;
@@ -877,4 +1358,10 @@ namespace glz
    template <uint32_t Format>
    struct serialize_partial
    {};
+
+   // Preset options for size-optimized builds (embedded systems)
+   struct opts_size : opts
+   {
+      glz::optimization_level optimization_level = glz::optimization_level::size;
+   };
 }
