@@ -3,6 +3,7 @@
 
 #pragma once
 
+#include "glaze/core/buffer_traits.hpp"
 #include "glaze/core/opts.hpp"
 #include "glaze/core/write.hpp"
 #include "glaze/core/write_chars.hpp"
@@ -38,7 +39,7 @@ namespace glz
    struct to<CSV, T>
    {
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&& ctx, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
          write_chars::op<Opts>(value, ctx, b, ix);
       }
@@ -48,13 +49,16 @@ namespace glz
    struct to<CSV, T>
    {
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&&, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
+         if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+            return;
+         }
          if (value) {
-            dump<'1'>(b, ix);
+            dump('1', b, ix);
          }
          else {
-            dump<'0'>(b, ix);
+            dump('0', b, ix);
          }
       }
    };
@@ -63,16 +67,22 @@ namespace glz
    struct to<CSV, T>
    {
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&& ctx, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
          if constexpr (resizable<T>) {
             if constexpr (check_layout(Opts) == rowwise) {
                const auto n = value.size();
                for (size_t i = 0; i < n; ++i) {
                   serialize<CSV>::op<Opts>(value[i], ctx, b, ix);
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
 
                   if (i != (n - 1)) {
-                     dump<','>(b, ix);
+                     if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                        return;
+                     }
+                     dump(',', b, ix);
                      if constexpr (is_output_streaming<B>) {
                         flush_buffer(b, ix);
                      }
@@ -87,9 +97,15 @@ namespace glz
             const auto n = value.size();
             for (size_t i = 0; i < n; ++i) {
                serialize<CSV>::op<Opts>(value[i], ctx, b, ix);
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
+               }
 
                if (i != (n - 1)) {
-                  dump<','>(b, ix);
+                  if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
+                  dump(',', b, ix);
                   if constexpr (is_output_streaming<B>) {
                      flush_buffer(b, ix);
                   }
@@ -107,7 +123,7 @@ namespace glz
       using Row = typename T::value_type;
 
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&& ctx, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
          if constexpr (check_layout(Opts) == rowwise) {
             // Write row by row
@@ -124,7 +140,10 @@ namespace glz
                   }
 
                   if (c < n_cols - 1) {
-                     dump<','>(b, ix);
+                     if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                        return;
+                     }
+                     dump(',', b, ix);
                      if constexpr (is_output_streaming<B>) {
                         flush_buffer(b, ix);
                      }
@@ -132,7 +151,10 @@ namespace glz
                }
 
                if (r < n_rows - 1) {
-                  dump<'\n'>(b, ix);
+                  if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
+                  dump('\n', b, ix);
                   if constexpr (is_output_streaming<B>) {
                      flush_buffer(b, ix);
                   }
@@ -141,7 +163,10 @@ namespace glz
 
             // Always add a trailing newline for consistency
             if (n_rows > 0) {
-               dump<'\n'>(b, ix);
+               if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                  return;
+               }
+               dump('\n', b, ix);
             }
          }
          else {
@@ -169,7 +194,10 @@ namespace glz
                   // else write empty cell (nothing to write)
 
                   if (r < value.size() - 1) {
-                     dump<','>(b, ix);
+                     if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                        return;
+                     }
+                     dump(',', b, ix);
                      if constexpr (is_output_streaming<B>) {
                         flush_buffer(b, ix);
                      }
@@ -177,7 +205,10 @@ namespace glz
                }
 
                if (c < max_cols - 1) {
-                  dump<'\n'>(b, ix);
+                  if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
+                  dump('\n', b, ix);
                   if constexpr (is_output_streaming<B>) {
                      flush_buffer(b, ix);
                   }
@@ -186,7 +217,10 @@ namespace glz
 
             // Add trailing newline for consistency
             if (max_cols > 0) {
-               dump<'\n'>(b, ix);
+               if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                  return;
+               }
+               dump('\n', b, ix);
             }
          }
       }
@@ -206,64 +240,61 @@ namespace glz
 
    // Dump a CSV string with proper quoting and escaping
    template <class B>
-   inline void dump_csv_string(const sv str, B& b, size_t& ix)
+   inline void dump_csv_string(is_context auto& ctx, const sv str, B& b, size_t& ix)
    {
       if (needs_csv_quoting(str)) {
-         // Need to quote this string
-         dump<'"'>(b, ix);
+         // Need to quote this string - worst case: every char is a quote (doubled) plus surrounding quotes
+         if (!ensure_space(ctx, b, ix + str.size() * 2 + 2 + write_padding_bytes)) [[unlikely]] {
+            return;
+         }
+
+         dump('"', b, ix);
 
          // Write the string, escaping quotes by doubling them
          for (const auto c : str) {
             if (c == '"') {
-               dump<'"'>(b, ix);
-               dump<'"'>(b, ix);
+               dump('"', b, ix);
+               dump('"', b, ix);
             }
             else {
-               if constexpr (vector_like<B>) {
-                  if (ix >= b.size()) [[unlikely]] {
-                     b.resize(2 * (ix + 1));
-                  }
-               }
                b[ix] = c;
                ++ix;
             }
          }
 
-         dump<'"'>(b, ix);
+         dump('"', b, ix);
       }
       else {
          // No special characters, write as-is
+         if (!ensure_space(ctx, b, ix + str.size() + write_padding_bytes)) [[unlikely]] {
+            return;
+         }
          dump_maybe_empty(str, b, ix);
       }
    }
 
    // Dump a single character with CSV quoting if needed
    template <class B>
-   inline void dump_csv_char(const char c, B& b, size_t& ix)
+   inline void dump_csv_char(is_context auto& ctx, const char c, B& b, size_t& ix)
    {
+      // Worst case: quoted double-quote = 4 chars (""")
+      if (!ensure_space(ctx, b, ix + 4 + write_padding_bytes)) [[unlikely]] {
+         return;
+      }
+
       if (c == ',' || c == '"' || c == '\n' || c == '\r') {
-         dump<'"'>(b, ix);
+         dump('"', b, ix);
          if (c == '"') {
-            dump<'"'>(b, ix);
-            dump<'"'>(b, ix);
+            dump('"', b, ix);
+            dump('"', b, ix);
          }
          else {
-            if constexpr (vector_like<B>) {
-               if (ix >= b.size()) [[unlikely]] {
-                  b.resize(2 * (ix + 1));
-               }
-            }
             b[ix] = c;
             ++ix;
          }
-         dump<'"'>(b, ix);
+         dump('"', b, ix);
       }
       else {
-         if constexpr (vector_like<B>) {
-            if (ix >= b.size()) [[unlikely]] {
-               b.resize(2 * (ix + 1));
-            }
-         }
          b[ix] = c;
          ++ix;
       }
@@ -274,13 +305,13 @@ namespace glz
    struct to<CSV, T>
    {
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&&, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
          if constexpr (char_t<T>) {
-            dump_csv_char(value, b, ix);
+            dump_csv_char(ctx, value, b, ix);
          }
          else {
-            dump_csv_string(value, b, ix);
+            dump_csv_string(ctx, value, b, ix);
          }
       }
    };
@@ -289,25 +320,37 @@ namespace glz
    struct to<CSV, T>
    {
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&& ctx, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
          if constexpr (check_layout(Opts) == rowwise) {
             for (auto& [name, data] : value) {
                if constexpr (check_use_headers(Opts)) {
+                  if (!ensure_space(ctx, b, ix + name.size() + 2 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
                   dump_maybe_empty(name, b, ix);
-                  dump<','>(b, ix);
+                  dump(',', b, ix);
                }
                const auto n = data.size();
                for (size_t i = 0; i < n; ++i) {
                   serialize<CSV>::op<Opts>(data[i], ctx, b, ix);
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
                   if (i < n - 1) {
-                     dump<','>(b, ix);
+                     if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                        return;
+                     }
+                     dump(',', b, ix);
                      if constexpr (is_output_streaming<B>) {
                         flush_buffer(b, ix);
                      }
                   }
                }
-               dump<'\n'>(b, ix);
+               if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                  return;
+               }
+               dump('\n', b, ix);
                if constexpr (is_output_streaming<B>) {
                   flush_buffer(b, ix);
                }
@@ -319,16 +362,22 @@ namespace glz
             if constexpr (check_use_headers(Opts)) {
                size_t i = 0;
                for (auto& [name, data] : value) {
+                  if (!ensure_space(ctx, b, ix + name.size() + 2 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
                   dump_maybe_empty(name, b, ix);
                   ++i;
                   if (i < n) {
-                     dump<','>(b, ix);
+                     dump(',', b, ix);
                      if constexpr (is_output_streaming<B>) {
                         flush_buffer(b, ix);
                      }
                   }
                }
-               dump<'\n'>(b, ix);
+               if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                  return;
+               }
+               dump('\n', b, ix);
                if constexpr (is_output_streaming<B>) {
                   flush_buffer(b, ix);
                }
@@ -345,9 +394,15 @@ namespace glz
                   }
 
                   serialize<CSV>::op<Opts>(data[row], ctx, b, ix);
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
                   ++i;
                   if (i < n) {
-                     dump<','>(b, ix);
+                     if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                        return;
+                     }
+                     dump(',', b, ix);
                      if constexpr (is_output_streaming<B>) {
                         flush_buffer(b, ix);
                      }
@@ -358,7 +413,10 @@ namespace glz
                   break;
                }
 
-               dump<'\n'>(b, ix);
+               if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                  return;
+               }
+               dump('\n', b, ix);
                if constexpr (is_output_streaming<B>) {
                   flush_buffer(b, ix);
                }
@@ -374,7 +432,7 @@ namespace glz
    struct to<CSV, T>
    {
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&& ctx, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
          static constexpr auto N = reflect<T>::size;
 
@@ -389,6 +447,9 @@ namespace glz
 
          if constexpr (check_layout(Opts) == rowwise) {
             for_each<N>([&]<auto I>() {
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
+               }
                using value_type = typename std::decay_t<refl_t<T, I>>::value_type;
 
                static constexpr sv key = reflect<T>::keys[I];
@@ -408,32 +469,53 @@ namespace glz
                   const auto size = member[0].size();
                   for (size_t i = 0; i < size; ++i) {
                      if constexpr (check_use_headers(Opts)) {
+                        if (!ensure_space(ctx, b, ix + key.size() + 32 + write_padding_bytes)) [[unlikely]] {
+                           return;
+                        }
                         dump<key>(b, ix);
-                        dump<'['>(b, ix);
+                        dump('[', b, ix);
                         write_chars::op<Opts>(i, ctx, b, ix);
-                        dump<']'>(b, ix);
-                        dump<','>(b, ix);
+                        dump(']', b, ix);
+                        dump(',', b, ix);
                      }
 
                      for (size_t j = 0; j < count; ++j) {
                         serialize<CSV>::op<Opts>(member[j][i], ctx, b, ix);
+                        if (bool(ctx.error)) [[unlikely]] {
+                           return;
+                        }
                         if (j != count - 1) {
-                           dump<','>(b, ix);
+                           if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                              return;
+                           }
+                           dump(',', b, ix);
                         }
                      }
 
                      if (i != size - 1) {
-                        dump<'\n'>(b, ix);
+                        if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                           return;
+                        }
+                        dump('\n', b, ix);
                      }
                   }
                }
                else {
                   if constexpr (check_use_headers(Opts)) {
+                     if (!ensure_space(ctx, b, ix + key.size() + 2 + write_padding_bytes)) [[unlikely]] {
+                        return;
+                     }
                      dump<key>(b, ix);
-                     dump<','>(b, ix);
+                     dump(',', b, ix);
                   }
                   serialize<CSV>::op<Opts>(get_member(value, mem), ctx, b, ix);
-                  dump<'\n'>(b, ix);
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
+                  if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
+                  dump('\n', b, ix);
                }
             });
          }
@@ -441,6 +523,9 @@ namespace glz
             // write titles
             if constexpr (check_use_headers(Opts)) {
                for_each<N>([&]<auto I>() {
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
                   using X = refl_t<T, I>;
 
                   static constexpr sv key = reflect<T>::keys[I];
@@ -457,12 +542,15 @@ namespace glz
                   if constexpr (fixed_array_value_t<X>) {
                      const auto size = get_member(value, member)[0].size();
                      for (size_t i = 0; i < size; ++i) {
+                        if (!ensure_space(ctx, b, ix + key.size() + 32 + write_padding_bytes)) [[unlikely]] {
+                           return;
+                        }
                         dump<key>(b, ix);
-                        dump<'['>(b, ix);
+                        dump('[', b, ix);
                         write_chars::op<Opts>(i, ctx, b, ix);
-                        dump<']'>(b, ix);
+                        dump(']', b, ix);
                         if (i != size - 1) {
-                           dump<','>(b, ix);
+                           dump(',', b, ix);
                         }
                      }
                   }
@@ -471,11 +559,17 @@ namespace glz
                   }
 
                   if constexpr (I != N - 1) {
-                     dump<','>(b, ix);
+                     if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                        return;
+                     }
+                     dump(',', b, ix);
                   }
                });
 
-               dump<'\n'>(b, ix);
+               if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                  return;
+               }
+               dump('\n', b, ix);
             }
 
             size_t row = 0;
@@ -483,6 +577,9 @@ namespace glz
 
             while (true) {
                for_each<N>([&]<auto I>() {
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
                   using X = std::decay_t<refl_t<T, I>>;
 
                   decltype(auto) mem = [&]() -> decltype(auto) {
@@ -504,8 +601,14 @@ namespace glz
                      const auto n = member[0].size();
                      for (size_t i = 0; i < n; ++i) {
                         serialize<CSV>::op<Opts>(member[row][i], ctx, b, ix);
+                        if (bool(ctx.error)) [[unlikely]] {
+                           return;
+                        }
                         if (i != n - 1) {
-                           dump<','>(b, ix);
+                           if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                              return;
+                           }
+                           dump(',', b, ix);
                         }
                      }
                   }
@@ -517,20 +620,29 @@ namespace glz
                      }
 
                      serialize<CSV>::op<Opts>(member[row], ctx, b, ix);
+                     if (bool(ctx.error)) [[unlikely]] {
+                        return;
+                     }
 
                      if constexpr (I != N - 1) {
-                        dump<','>(b, ix);
+                        if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                           return;
+                        }
+                        dump(',', b, ix);
                      }
                   }
                });
 
-               if (end) {
+               if (end || bool(ctx.error)) {
                   break;
                }
 
                ++row;
 
-               dump<'\n'>(b, ix);
+               if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                  return;
+               }
+               dump('\n', b, ix);
             }
          }
       }
@@ -544,27 +656,45 @@ namespace glz
       using U = typename T::value_type;
 
       template <auto Opts, class B>
-      static void op(auto&& value, is_context auto&& ctx, B&& b, auto&& ix)
+      static void op(auto&& value, is_context auto&& ctx, B&& b, auto& ix)
       {
          static constexpr auto N = reflect<U>::size;
 
          // Write headers (field names) if enabled
          if constexpr (check_use_headers(Opts)) {
             for_each<N>([&]<auto I>() {
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
+               }
                static constexpr sv key = reflect<U>::keys[I];
                serialize<CSV>::op<Opts>(key, ctx, b, ix);
 
                if constexpr (I < N - 1) {
-                  dump<','>(b, ix);
+                  if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
+                  dump(',', b, ix);
                }
             });
 
-            dump<'\n'>(b, ix);
+            if (bool(ctx.error)) [[unlikely]] {
+               return;
+            }
+            if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+               return;
+            }
+            dump('\n', b, ix);
          }
 
          // Write each struct as a row
          for (const auto& item : value) {
+            if (bool(ctx.error)) [[unlikely]] {
+               return;
+            }
             for_each<N>([&]<auto I>() {
+               if (bool(ctx.error)) [[unlikely]] {
+                  return;
+               }
                decltype(auto) mem = [&]() -> decltype(auto) {
                   if constexpr (reflectable<U>) {
                      return get<I>(to_tie(item));
@@ -577,11 +707,20 @@ namespace glz
                serialize<CSV>::op<Opts>(get_member(item, mem), ctx, b, ix);
 
                if constexpr (I < N - 1) {
-                  dump<','>(b, ix);
+                  if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+                     return;
+                  }
+                  dump(',', b, ix);
                }
             });
 
-            dump<'\n'>(b, ix);
+            if (bool(ctx.error)) [[unlikely]] {
+               return;
+            }
+            if (!ensure_space(ctx, b, ix + 1 + write_padding_bytes)) [[unlikely]] {
+               return;
+            }
+            dump('\n', b, ix);
          }
       }
    };
