@@ -9,6 +9,7 @@
 #include <charconv>
 #include <chrono>
 #include <condition_variable>
+#include <cstring>
 #include <expected>
 #include <functional>
 #include <future>
@@ -1344,6 +1345,7 @@ namespace glz
       {
          std::shared_ptr<socket_type> socket;
          std::shared_ptr<asio::streambuf> buffer;
+         std::string body;
          asio::ip::tcp::endpoint remote_endpoint;
          std::shared_ptr<asio::steady_timer> idle_timer;
          uint32_t request_count = 0;
@@ -1612,34 +1614,30 @@ namespace glz
          }
 
          if (content_length > 0) {
-            std::string body;
-            body.reserve(content_length);
-            // Append what's already in the buffer
+            conn->body.resize(content_length);
+            // Copy what's already in the buffer to the body
             const size_t initial_body_size = std::min(content_length, conn->buffer->size());
-            body.append(static_cast<const char*>(conn->buffer->data().data()), initial_body_size);
+            const auto missing_bytes = content_length - initial_body_size;
+            std::memcpy(conn->body.data(), conn->buffer->data().data(), initial_body_size);
             conn->buffer->consume(initial_body_size);
-
-            if (body.length() < content_length) {
-               asio::async_read(*conn->socket, *conn->buffer, asio::transfer_exactly(content_length - body.length()),
-                                [this, conn, method_opt, target, headers, body = std::move(body)](std::error_code ec,
-                                                                                                  size_t) mutable {
+            if (initial_body_size < content_length) {
+               asio::async_read(*conn->socket, asio::buffer(&conn->body[initial_body_size], missing_bytes),
+                                asio::transfer_exactly(missing_bytes),
+                                [this, conn, method_opt, target, headers](std::error_code ec, size_t) {
                                    if (ec) {
                                       error_handler(ec, std::source_location::current());
                                       return;
                                    }
                                    // Append newly read data
-                                   body.append(static_cast<const char*>(conn->buffer->data().data()),
-                                               conn->buffer->size());
-                                   conn->buffer->consume(conn->buffer->size());
-                                   process_full_request_with_conn(conn, *method_opt, target, headers, std::move(body));
+                                   process_full_request_with_conn(conn, *method_opt, target, headers);
                                 });
             }
             else {
-               process_full_request_with_conn(conn, *method_opt, target, headers, std::move(body));
+               process_full_request_with_conn(conn, *method_opt, target, headers);
             }
          }
          else {
-            process_full_request_with_conn(conn, *method_opt, target, headers, "");
+            process_full_request_with_conn(conn, *method_opt, target, headers);
          }
       }
 
@@ -1679,9 +1677,9 @@ namespace glz
       // Process a full request using connection state (new keep-alive aware version)
       inline void process_full_request_with_conn(std::shared_ptr<connection_state> conn, http_method method,
                                                  const std::string& target,
-                                                 const std::unordered_map<std::string, std::string>& headers,
-                                                 std::string body)
+                                                 const std::unordered_map<std::string, std::string>& headers)
       {
+         std::string body = std::move(conn->body);
          // Check for a streaming handler first
          auto handler_it = streaming_handlers_.find(target);
          if (handler_it != streaming_handlers_.end()) {
@@ -1698,13 +1696,18 @@ namespace glz
          request request;
          request.method = method;
          request.target = target;
-         request.headers = headers;
-         request.body = std::move(body);
          request.remote_ip = conn->remote_endpoint.address().to_string();
          request.remote_port = conn->remote_endpoint.port();
 
+         // Parse path and query string from target
+         const auto [path_view, query_string] = split_target(target);
+         request.path = std::string(path_view);
+         request.query = parse_urlencoded(query_string);
+         request.headers = headers;
+         request.body = std::move(body);
+
          // Find a matching route
-         auto [handle, params] = root_router.match(method, target);
+         auto [handle, params] = root_router.match(method, request.path);
 
          // Create the response object
          response response;
@@ -2080,13 +2083,18 @@ namespace glz
          request request;
          request.method = method;
          request.target = target;
-         request.headers = headers;
-         request.body = std::move(body);
          request.remote_ip = remote_endpoint.address().to_string();
          request.remote_port = remote_endpoint.port();
 
+         // Parse path and query string from target
+         const auto [path_view, query_string] = split_target(target);
+         request.path = std::string(path_view);
+         request.query = parse_urlencoded(query_string);
+         request.headers = headers;
+         request.body = std::move(body);
+
          // Find a matching route using http_router::match which handles both exact and parameterized routes
-         auto [handle, params] = root_router.match(method, target);
+         auto [handle, params] = root_router.match(method, request.path);
 
          // Create the response object up front so we can reuse it in fallback flows
          response response;
